@@ -9,8 +9,8 @@
 
 #define print_usage_and_exit() die ("Usage: %s [-v] [-b filename] device\n\
 -v		prints blocks number of every block unpacked\n\
--b filename	makes unpack to save bitmap of blocks unpacked\n\
--j filename     makes unpack to store journal in specified file\n", argv[0]);
+-b filename	saves bitmap of blocks unpacked to filename\n\
+-j filename     stores journal in the filename\n", argv[0]);
 
 
 /* when super block gets unpacked for the first time - create a bitmap
@@ -583,43 +583,28 @@ out:
 }
 
 
-int main (int argc, char ** argv)
-{
+int do_unpack(char *host, char *j_filename, char *filename, int verbose) {
     int fd, fdj = -2;
     int c;
-    char * j_filename;
-    char * filename = ".bitmap";
-    struct rlimit lim = {0xffffffff, 0xffffffff};
-
-    print_banner ("unpack");
+    struct rlimit lim = {RLIM_INFINITY, RLIM_INFINITY};
 
+    if (filename == NULL)
+	    filename = ".bitmap";
+    
+    if (j_filename)
+	    Default_journal = 0;
+    
     /* with this 2.4.0-test9's file_write does not send SIGXFSZ */
     if (setrlimit (RLIMIT_FSIZE, &lim)) {
 	fprintf  (stderr, "sertlimit failed: %m\n");
     }
 
-    while ((c = getopt (argc, argv, "vb:j:")) != EOF) {
-	switch (c) {
-	case 'v':
-	    verbose = 1;
-	    break;
-	case 'b':
-	    asprintf (&filename, "%s", optarg);
-	    break;
-	case 'j':
-	    Default_journal = 0;
-	    asprintf (&j_filename, "%s", optarg);
-	    break;    
-	}
+    if (misc_device_mounted(host) > 0) {
+	fprintf(stderr, "%s seems mounted, umount it first\n", host);
+	return 0;
     }
-    if (optind != argc - 1)
-	/* only one non-option argument is permitted */
-	print_usage_and_exit();
-
-    if (misc_device_mounted(argv[optind], 0))
-	reiserfs_panic ("%s seems mounted, umount it first\n", argv[optind]);
   
-    fd = open (argv[optind], O_RDWR | O_LARGEFILE);
+    fd = open (host, O_RDWR | O_LARGEFILE);
     if (fd == -1) {
 	perror ("open failed");
 	return 0;
