@@ -1,6 +1,8 @@
 /*
- * Copyright 1996-2003 Hans Reiser
+ * Copyright 1996-2004 by Hans Reiser, licensing governed by 
+ * reiserfsprogs/README
  */
+
 #include "fsck.h"
 
 static int do_items_have_the_same_type (struct item_head * ih, struct key * key)
@@ -177,7 +179,8 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
     struct key * next_key, * key;
     __u32 sd_first_direct_byte = 0;
     __u64 sd_size;
-    int retval, i, was_tail = 0;
+    unsigned int i;
+    int retval, was_tail = 0;
     int had_direct = 0;
     int key_version = get_ih_key_format (sd_ih);
     int next_is_another_object = 0;
@@ -222,7 +225,7 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 	    }
 	
 	    // does not change path
-	    next_key = get_next_key_2 (&path);
+	    next_key = reiserfs_next_key(&path);
 
 	    if (next_key == 0 || not_of_one_file (key, next_key) || 
 		(!is_indirect_key (next_key) && !is_direct_key(next_key))) 
@@ -255,12 +258,13 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 			one_more_corruption (fs, FIXABLE);
 		    } else {
 			fsck_log("are_file_items_correct: vpf-10280: block %lu, item (%d): The item format (%H)"\
-			    " is not equal to SD format (%d) - item was deleted\n",
+			    " is not equal to SD format (%d) - fixed.\n",
 			    get_bh(&path)->b_blocknr, PATH_LAST_POSITION(&path), ih, key_version);
 		
-			pathrelse (&path);
-			delete_N_items_after_key (key, NULL, 0 /* do not skip dir items */, 0);
-			return 1;
+
+			set_type_and_offset (key_version, &ih->ih_key, get_offset (&ih->ih_key), get_type (&ih->ih_key));
+			set_ih_key_format(ih, key_version);
+			mark_buffer_dirty(get_bh(&path));
 		    }
 		}
 
@@ -278,8 +282,8 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 			    fsck_log("are_file_items_correct: vpf-10732: block %lu, item (%d): The symlink format (%H)"\
 				" is not equal to 3.5 format (%d)\n", get_bh(&path)->b_blocknr, PATH_LAST_POSITION(&path), 
 				ih, KEY_FORMAT_1);
-			    set_ih_key_format(ih, KEY_FORMAT_1);
 			    set_type_and_offset(KEY_FORMAT_1, &ih->ih_key, get_offset(&ih->ih_key), get_type(&ih->ih_key));
+			    set_ih_key_format(ih, KEY_FORMAT_1);
 			    mark_buffer_dirty(get_bh(&path));
 			}
 			
@@ -330,10 +334,9 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 			*/
 			if (ind [I_UNFM_NUM (ih) - 1] == 0)
 			    *blocks += (fs->fs_blocksize >> 9);
-			
-			/* PATH is released here. */
+
+			/* path is released here. */
 			sd_first_direct_byte = indirect_to_direct (&path, sd_size - last_unfm_offset + 1, *symlink);
-			
 			/* last item of the file is direct item */
 			set_offset (key_version, key, sd_first_direct_byte);
 			set_type (key_version, key, TYPE_DIRECT);
@@ -367,7 +370,7 @@ int are_file_items_correct (struct item_head * sd_ih, void * sd, __u64 * size, _
 		    fsck_log ("are_file_items_correct: "
 			      "item must be indirect and must have invalid free space (%H)", ih);
 	
-                if (fsck_mode (fs) != FSCK_CHECK && fsck_mode (fs) != FSCK_FIX_FIXABLE)
+                if (fsck_mode (fs) == FSCK_REBUILD)
                 {		
                     set_ih_free_space(ih, 0);
                     mark_buffer_dirty (PATH_PLAST_BUFFER (&path));
@@ -525,7 +528,8 @@ static int create_first_item_of_file (struct item_head * ih, char * item, struct
     __u32 * ni = 0;
     struct buffer_head * unbh;
     struct item_head indih;
-    int retval, i;
+    unsigned int i;
+    int retval;
 
     mark_item_unreachable (&indih);
 
@@ -704,7 +708,7 @@ static void direct2indirect2 (unsigned long unfm, struct path * path)
 
 
 static int append_to_unformatted_node (struct item_head * comingih, struct item_head * ih, char * item,
-                                        struct path * path, int coming_len)
+                                        struct path * path, unsigned int coming_len)
 {
     struct buffer_head * bh, * unbh = NULL;
     __u64 end_of_data, free_space;
@@ -782,8 +786,7 @@ int reiserfsck_append_file (struct item_head * comingih, char * item, int pos, s
 	reiserfs_panic ("reiserfsck_append_file: Operation is not allowed for non-indirect item %k", &ih->ih_key);
 
     if (is_direct_ih (comingih)) {
-	//__u64 coming_len = get_bytes_number (0,comingih, item, CHECK_FREE_BYTES);
-	int coming_len = get_bytes_number (comingih, fs->fs_blocksize);
+	unsigned int coming_len = get_bytes_number (comingih, fs->fs_blocksize);
 
 	if (get_offset (&comingih->ih_key) < get_offset (&ih->ih_key) + fs->fs_blocksize * I_UNFM_NUM (ih)) {
 	    /* direct item fits to free space of indirect item */
@@ -834,7 +837,7 @@ int reiserfsck_append_file (struct item_head * comingih, char * item, int pos, s
     return retval;
 }
 
-int must_there_be_a_hole (struct item_head * comingih, struct path * path)
+long long int must_there_be_a_hole (struct item_head * comingih, struct path * path)
 {
     struct item_head * ih = PATH_PITEM_HEAD (path);
 
@@ -854,10 +857,10 @@ int must_there_be_a_hole (struct item_head * comingih, struct path * path)
 }
 
 
-int reiserfs_append_zero_unfm_ptr (struct path * path, int p_count)
+int reiserfs_append_zero_unfm_ptr (struct path * path, long long int p_count)
 {
     __u32 * ni;
-    int count;
+    long long int count;
 
     if (is_direct_ih (PATH_PITEM_HEAD (path)))
 	/* convert direct item to indirect */
@@ -1077,10 +1080,9 @@ static int reiserfsck_overwrite_file (struct item_head * comingih, char * item,
 int reiserfsck_file_write (struct item_head * ih, char * item, int was_in_tree) {
     struct path path;
     int count, pos_in_coming_item;
-    int retval, written;
+    long long int retval, written;
     struct key key;
     int file_format = KEY_FORMAT_UNDEFINED;
-    int symlink = 0;
     int relocated = 0;
 
     if (!was_in_tree) {
@@ -1102,14 +1104,16 @@ check_again:
         /*SD found*/
         file_format = get_ih_key_format (get_ih(&path));
         get_sd_mode (get_ih(&path), get_item(&path), &mode);
-        symlink = S_ISLNK(mode);
 
-        if (!symlink && file_format != get_ih_key_format (ih)) {
-            fsck_log ("vpf-10270: The file we are inserting the new item into has the different"
-		" format then found StatData, insertion was skipped\n\tSD: (%H)\n\tNew:(%H)\n", 
-		ih, get_ih(&path));
-            pathrelse (&path);
-            return 0;
+        if (file_format != get_ih_key_format (ih)) {
+	    /* Not for symlinks and not for items which should be relocted. */
+	    if (((S_ISDIR(mode) && is_direntry_ih(ih)) || 
+		(!S_ISDIR(mode) && !is_direntry_ih(ih))) && 
+		!S_ISLNK(mode))
+	    {
+		set_type_and_offset (file_format, &ih->ih_key, get_offset (&ih->ih_key), get_type (&ih->ih_key));
+		set_ih_key_format(ih, file_format);		
+	    }
         }
 
 	if (!relocated && should_relocate (ih)) {
