@@ -29,6 +29,9 @@
 
 */
 
+#ifndef REISERFSPROGS_FS_H
+#define REISERFSPRIGS_FS_H 
+
 #ifndef NO_EXTERN_INLINE
 # define extern_inline extern inline
 #else
@@ -1560,6 +1563,7 @@ extern unsigned int get_journal_new_start_must (reiserfs_filsys_t * fs);
 extern unsigned int get_journal_start_must (reiserfs_filsys_t * fs);
 /*extern hashf_t hashes [];*/
 
+#endif
 
 /*
  * Local variables:
